"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils = require("tns-core-modules/utils/utils");
var application_1 = require("tns-core-modules/application");
var worker = {};
var eventWorker = {};
var Particle = (function () {
    function Particle() {
        this.eventIds = new Map();
        io.particle.android.sdk.cloud.ParticleCloudSDK.init(utils.ad.getApplicationContext());
        this.tokens = {};
    }
    Particle.prototype.initWorkerIfNeeded = function (productId) {
        if (!worker[productId]) {
            if (global["TNS_WEBPACK"]) {
                var WorkerScript = require("nativescript-worker-loader!./particle-worker.js");
                worker[productId] = new WorkerScript();
            }
            else {
                worker[productId] = new Worker("./particle-worker.js");
            }
        }
    };
    Particle.prototype.initEventWorkerIfNeeded = function (productId) {
        var _this = this;
        if (!eventWorker[productId]) {
            if (global["TNS_WEBPACK"]) {
                var EventWorkerScript = require("nativescript-worker-loader!./particle-event-worker.js");
                eventWorker[productId] = new EventWorkerScript();
            }
            else {
                eventWorker[productId] = new Worker("./particle-event-worker.js");
            }
            eventWorker[productId].onmessage = function (msg) {
                if (msg.data.success) {
                    var handlerId = msg.data.handlerId;
                    var handler = _this.eventIds.get(handlerId);
                    handler && handler(msg.data.data);
                }
                else {
                }
            };
        }
    };
    Particle.prototype.login = function (options) {
        this.initWorkerIfNeeded('common');
        return new Promise(function (resolve, reject) {
            worker['common'].postMessage({
                action: "login",
                options: options
            });
            worker['common'].onmessage = function (msg) { return msg.data.success ? resolve() : reject(msg.data.error); };
        });
    };
    Particle.prototype.loginWithToken = function (token, productId) {
        if (productId) {
            this.tokens[productId] = token;
            this.currentProduct = productId;
        }
        io.particle.android.sdk.cloud.ParticleCloudSDK.getCloud().setAccessToken(token);
    };
    Particle.prototype.setOAuthConfig = function (id, secret) {
        console.log("'setOAuthConfig' is not currently implemented on Android. Feel like doing a PR? :)");
    };
    Particle.prototype.logout = function () {
        io.particle.android.sdk.cloud.ParticleCloudSDK.getCloud().logOut();
        Object.keys(worker).forEach(function (e) { return worker[e].terminate(); });
        Object.keys(eventWorker).forEach(function (e) { return eventWorker[e].terminate(); });
        eventWorker = {};
        worker = {};
    };
    Particle.prototype.publish = function (name, data, isPrivate, ttl, productId) {
        var _this = this;
        if (ttl === void 0) { ttl = 60; }
        return new Promise(function (resolve, reject) {
            _this.authIfNeeded(productId);
            _this.initWorkerIfNeeded(productId);
            worker[productId].postMessage({
                action: "publish",
                options: {
                    name: name,
                    data: data,
                    isPrivate: isPrivate,
                    ttl: ttl
                }
            });
            resolve();
        });
    };
    Particle.prototype.subscribe = function (prefix, eventHandler, productId) {
        if (this.eventIds.has(prefix)) {
            console.log("Already subscribed for prefix '" + prefix + "' - not registering another event handler.");
            return;
        }
        this.eventIds.set(prefix, eventHandler);
        this.authIfNeeded(productId);
        this.initEventWorkerIfNeeded(productId);
        eventWorker[productId].postMessage({
            action: "subscribe",
            options: {
                handlerId: prefix,
                prefix: prefix
            }
        });
    };
    Particle.prototype.unsubscribe = function (prefix, productId) {
        if (!this.eventIds.has(prefix)) {
            console.log("No handler registered from prefix '" + prefix + "' - skipping unsubscribe");
            return;
        }
        this.authIfNeeded(productId);
        this.initEventWorkerIfNeeded(productId);
        eventWorker[productId].postMessage({
            action: "unsubscribe",
            options: {
                prefix: prefix
            }
        });
        this.eventIds.delete(prefix);
    };
    Particle.prototype.listDevices = function (productId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.authIfNeeded(productId);
            _this.initWorkerIfNeeded(productId);
            _this.initEventWorkerIfNeeded(productId);
            worker[productId].postMessage({
                action: "listDevices"
            });
            worker[productId].onmessage = function (msg) {
                if (msg.data.success) {
                    var devices = msg.data.devices;
                    devices.map(function (device) {
                        device.rename = function (name) { return _this.renameDevice(device.id, name, device.productId); };
                        device.unclaim = function () { return _this.unclaimDevice(device.id, device.productId); };
                        device.callFunction = function (name, args) { return _this.callFunction(device.id, name, args, device.productId); };
                        device.getVariable = function (name) { return _this.getVariable(device.id, name, device.productId); };
                        device.subscribe = function (prefix, eventHandler) { return _this.subscribeDevice(device.id, prefix, eventHandler, device.productId); };
                        device.unsubscribe = function (prefix) { return _this.unsubscribeDevice(device.id, prefix, device.productId); };
                    });
                    resolve(devices);
                }
                else {
                    reject(msg.data.error);
                }
            };
        });
    };
    Particle.prototype.isAuthenticated = function () {
        return io.particle.android.sdk.cloud.ParticleCloudSDK.getCloud().isLoggedIn();
    };
    Particle.prototype.accessToken = function () {
        return io.particle.android.sdk.cloud.ParticleCloudSDK.getCloud().getAccessToken();
    };
    Particle.prototype.startDeviceSetupWizard = function (productId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.authIfNeeded(productId);
            var intent = application_1.android.foregroundActivity.getIntent();
            io.particle.android.sdk.devicesetup.ParticleDeviceSetupLibrary.init(application_1.android.foregroundActivity);
            var builder = new io.particle.android.sdk.devicesetup.SetupCompleteIntentBuilder({
                buildIntent: function (context, setupResult) {
                    resolve(setupResult.wasSuccessful());
                    return intent;
                }
            });
            io.particle.android.sdk.devicesetup.ParticleDeviceSetupLibrary.startDeviceSetup(application_1.android.foregroundActivity, builder);
        });
    };
    Particle.prototype.getDeviceSetupCustomizer = function () {
    };
    Particle.prototype.renameDevice = function (deviceId, name, productId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.authIfNeeded(productId);
            _this.initWorkerIfNeeded(productId);
            worker[productId].postMessage({
                action: "rename",
                options: {
                    deviceId: deviceId,
                    name: name
                }
            });
            worker[productId].onmessage = function (msg) { return msg.data.success ? resolve() : reject(msg.data.error); };
        });
    };
    Particle.prototype.unclaimDevice = function (deviceId, productId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.authIfNeeded(productId);
            _this.initWorkerIfNeeded(productId);
            worker[productId].postMessage({
                action: "unclaim",
                options: {
                    deviceId: deviceId
                }
            });
            worker[productId].onmessage = function (msg) { return msg.data.success ? resolve() : reject(msg.data.error); };
        });
    };
    Particle.prototype.callFunction = function (deviceId, name, args, productId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.authIfNeeded(productId);
            _this.initWorkerIfNeeded(productId);
            worker[productId].postMessage({
                action: "callFunction",
                options: {
                    deviceId: deviceId,
                    name: name,
                    args: args
                }
            });
            worker[productId].onmessage = function (msg) { return msg.data.success ? resolve(msg.data.result) : reject(msg.data.error); };
        });
    };
    Particle.prototype.getVariable = function (deviceId, name, productId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.authIfNeeded(productId);
            _this.initWorkerIfNeeded(productId);
            worker[productId].postMessage({
                action: "getVariable",
                options: {
                    deviceId: deviceId,
                    name: name
                }
            });
            worker[productId].onmessage = function (msg) { return msg.data.success ? resolve(msg.data.result) : reject(msg.data.error); };
        });
    };
    Particle.prototype.subscribeDevice = function (deviceId, prefix, eventHandler, productId) {
        var handlerId = deviceId + "_" + prefix;
        if (this.eventIds.has(handlerId)) {
            console.log("Already subscribed for prefix '" + prefix + "' - not registering another event handler.");
            return;
        }
        this.eventIds.set(handlerId, eventHandler);
        this.authIfNeeded(productId);
        this.initEventWorkerIfNeeded(productId);
        eventWorker[productId].postMessage({
            action: "subscribeDevice",
            options: {
                handlerId: handlerId,
                deviceId: deviceId,
                prefix: prefix
            }
        });
    };
    Particle.prototype.unsubscribeDevice = function (deviceId, prefix, productId) {
        var handlerId = deviceId + "_" + prefix;
        if (!this.eventIds.has(handlerId)) {
            console.log("No handler registered from prefix '" + handlerId + "' - skipping unsubscribe");
            return;
        }
        this.authIfNeeded(productId);
        this.initEventWorkerIfNeeded(productId);
        eventWorker[productId].postMessage({
            action: "unsubscribeDevice",
            options: {
                handlerId: handlerId,
                deviceId: deviceId,
                prefix: prefix,
            }
        });
        this.eventIds.delete(handlerId);
    };
    Particle.prototype.authIfNeeded = function (productId) {
        if (productId && this.tokens[productId] && this.currentProduct !== productId) {
            io.particle.android.sdk.cloud.ParticleCloudSDK.getCloud().setAccessToken(this.tokens[productId]);
            this.currentProduct = productId;
        }
    };
    return Particle;
}());
exports.Particle = Particle;
